"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    }
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var BaseTask_1 = require("./BaseTask");
var GuardEvaluator_1 = require("./GuardEvaluator");
var TaskStatus_1 = require("./TaskStatus");
/** The behavior tree itself. */
var BaseBehaviorTree = /** @class */ (function (_super) {
    __extends(BaseBehaviorTree, _super);
    function BaseBehaviorTree(rootTask, object) {
        var _this = _super.call(this) || this;
        _this.rootTask = rootTask;
        _this.object = object;
        _this.tree = _this;
        _this.guardEvaluator = new GuardEvaluator_1.GuardEvaluator(_this);
        return _this;
    }
    BaseBehaviorTree.prototype.addChild = function (child) {
        if (this.rootTask !== undefined) {
            throw new Error('A behavior tree cannot have more than one root task.');
        }
        this.rootTask = child;
        return this;
    };
    BaseBehaviorTree.prototype.getChildCount = function () {
        return this.rootTask === undefined ? 0 : 1;
    };
    BaseBehaviorTree.prototype.getChild = function (index) {
        if (index === 0 && this.rootTask !== undefined) {
            return this.rootTask;
        }
        throw new Error("Index cannot be >= size: " + index + " >= " + this.getChildCount() + ".");
    };
    BaseBehaviorTree.prototype.getObject = function () {
        if (this.object === undefined) {
            throw new Error('Invalid state.');
        }
        return this.object;
    };
    BaseBehaviorTree.prototype.setObject = function (object) {
        this.object = object;
    };
    BaseBehaviorTree.prototype.childSuccess = function (runningTask) {
        this.success();
    };
    BaseBehaviorTree.prototype.childFail = function (runningTask) {
        this.fail();
    };
    BaseBehaviorTree.prototype.childRunning = function (runningTask, reporter) {
        this.running();
    };
    BaseBehaviorTree.prototype.step = function () {
        var task = this.rootTask;
        if (task === undefined) {
            throw new Error('Invalid state.');
        }
        if (task.getStatus() === TaskStatus_1.TaskStatus.Running) {
            task.run();
        }
        else {
            task.setControl(this);
            task.start();
            if (task.checkGuard(this)) {
                task.run();
            }
            else {
                task.fail();
            }
        }
    };
    BaseBehaviorTree.prototype.run = function () {
    };
    BaseBehaviorTree.prototype.resetTask = function () {
        _super.prototype.resetTask.call(this);
        // Re-assign tree.
        this.tree = this;
    };
    BaseBehaviorTree.prototype.reset = function () {
        _super.prototype.reset.call(this);
        this.rootTask = undefined;
        this.object = undefined;
    };
    return BaseBehaviorTree;
}(BaseTask_1.BaseTask));
exports.BaseBehaviorTree = BaseBehaviorTree;
