"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    }
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var BaseTask_1 = require("./BaseTask");
var TaskStatus_1 = require("./TaskStatus");
/**
 * A decorator is a wrapper that provides custom behavior for its child.
 * The child can be of any kind (branch task, leaf task or another decorator).
 */
var Decorator = /** @class */ (function (_super) {
    __extends(Decorator, _super);
    function Decorator(child) {
        var _this = _super.call(this) || this;
        _this.child = child;
        return _this;
    }
    Decorator.prototype.addChild = function (child) {
        if (this.child !== undefined) {
            throw new Error('A decorator task cannot have more than once child.');
        }
        this.child = child;
        return this;
    };
    Decorator.prototype.getChildCount = function () {
        return this.child === undefined ? 0 : 1;
    };
    Decorator.prototype.getChild = function (index) {
        if (index === 0 && this.child !== undefined) {
            return this.child;
        }
        throw new Error("Index cannot be >= size: " + index + " >= " + this.getChildCount() + ".");
    };
    Decorator.prototype.run = function () {
        var child = this.child;
        if (child === undefined) {
            throw new Error('This decorator has no child to run.');
        }
        if (child.getStatus() === TaskStatus_1.TaskStatus.Running) {
            child.run();
        }
        else {
            child.setControl(this);
            child.start();
            if (child.checkGuard(this)) {
                child.run();
            }
            else {
                child.fail();
            }
        }
    };
    Decorator.prototype.childSuccess = function (runningTask) {
        this.success();
    };
    Decorator.prototype.childFail = function (runningTask) {
        this.fail();
    };
    Decorator.prototype.childRunning = function (runningTask, reporter) {
        this.running();
    };
    Decorator.prototype.reset = function () {
        _super.prototype.reset.call(this);
        this.child = undefined;
    };
    return Decorator;
}(BaseTask_1.BaseTask));
exports.Decorator = Decorator;
