"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    }
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var BaseTask_1 = require("./BaseTask");
var TaskStatus_1 = require("./TaskStatus");
/**
 * A LeafTask is a terminal task of a behavior tree,
 * contains action or condition logic, can not have any child.
 */
var LeafTask = /** @class */ (function (_super) {
    __extends(LeafTask, _super);
    function LeafTask() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    LeafTask.prototype.run = function () {
        var status = this.execute();
        if (status === TaskStatus_1.TaskStatus.Succeeded) {
            this.success();
            return;
        }
        if (status === TaskStatus_1.TaskStatus.Failed) {
            this.fail();
            return;
        }
        if (status === TaskStatus_1.TaskStatus.Running) {
            this.running();
            return;
        }
        throw new Error('Invalid status returned by the execute method.');
    };
    LeafTask.prototype.addChild = function (child) {
        throw new Error('A leaf task cannot have any child.');
    };
    LeafTask.prototype.getChildCount = function () {
        return 0;
    };
    LeafTask.prototype.getChild = function (index) {
        throw new Error("A leaf task doesn't have any child.");
    };
    LeafTask.prototype.childSuccess = function (runningTask) {
        // No child.
    };
    LeafTask.prototype.childFail = function (runningTask) {
        // No child.
    };
    LeafTask.prototype.childRunning = function (runningTask, reporter) {
        // No child.
    };
    return LeafTask;
}(BaseTask_1.BaseTask));
exports.LeafTask = LeafTask;
