import { BranchTask } from "../BranchTask";
import { Task } from "../Task";
/**
 * A DynamicGuardSelector is a branch task that executes the first child
 * whose guard is evaluated to true. At every AI cycle, the children's guards
 * are re-evaluated, so if the guard of the running child is evaluated to false,
 * it is cancelled, and the child with the highest priority starts running.
 * The DynamicGuardSelector task finishes when no guard is evaluated to true
 * (thus failing) or when its active child finishes (returning the active child's
 * termination status).
 */
export declare class DynamicGuardSelector<T> extends BranchTask<T> {
    protected runningChild?: Task;
    childSuccess(runningTask: Task): void;
    childFail(runningTask: Task): void;
    childRunning(runningTask: Task, reporter: Task): void;
    run(): void;
    resetTask(): void;
    reset(): void;
}
