"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    }
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var BranchTask_1 = require("../BranchTask");
/**
 * A DynamicGuardSelector is a branch task that executes the first child
 * whose guard is evaluated to true. At every AI cycle, the children's guards
 * are re-evaluated, so if the guard of the running child is evaluated to false,
 * it is cancelled, and the child with the highest priority starts running.
 * The DynamicGuardSelector task finishes when no guard is evaluated to true
 * (thus failing) or when its active child finishes (returning the active child's
 * termination status).
 */
var DynamicGuardSelector = /** @class */ (function (_super) {
    __extends(DynamicGuardSelector, _super);
    function DynamicGuardSelector() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    DynamicGuardSelector.prototype.childSuccess = function (runningTask) {
        this.runningChild = undefined;
        this.success();
    };
    DynamicGuardSelector.prototype.childFail = function (runningTask) {
        this.runningChild = undefined;
        this.fail();
    };
    DynamicGuardSelector.prototype.childRunning = function (runningTask, reporter) {
        this.runningChild = runningTask;
        // Return a running status when a child says it's running.
        this.running();
    };
    DynamicGuardSelector.prototype.run = function () {
        var childToRun;
        for (var i = 0, n = this.children.length; i < n; ++i) {
            var child = this.children[i];
            if (child.checkGuard(this)) {
                childToRun = child;
                break;
            }
        }
        if (this.runningChild !== undefined && this.runningChild !== childToRun) {
            this.runningChild.cancel();
            this.runningChild = undefined;
        }
        if (childToRun === undefined) {
            this.fail();
        }
        else {
            if (this.runningChild === undefined) {
                this.runningChild = childToRun;
                this.runningChild.setControl(this);
                this.runningChild.start();
            }
            this.runningChild.run();
        }
    };
    DynamicGuardSelector.prototype.resetTask = function () {
        _super.prototype.resetTask.call(this);
        this.runningChild = undefined;
    };
    DynamicGuardSelector.prototype.reset = function () {
        _super.prototype.reset.call(this);
        this.runningChild = undefined;
    };
    return DynamicGuardSelector;
}(BranchTask_1.BranchTask));
exports.DynamicGuardSelector = DynamicGuardSelector;
