"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    }
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var BranchTask_1 = require("../BranchTask");
var ParallelOrchestratorResume_1 = require("./ParallelOrchestratorResume");
var ParallelPolicySequence_1 = require("./ParallelPolicySequence");
var ParallelResult_1 = require("./ParallelResult");
/**
 * A Parallel is a special branch task that runs all children when stepped.
 *
 * Its actual behavior depends on its orchestrator and policy.
 *
 * The execution of the parallel task's children depends on its orchestrator:
 * - Orchestrator.Resume: the parallel task restarts or runs each child every step.
 * - Orchestrator.Join: child tasks will run until success or failure but will not
 * re-run until the parallel task has succeeded or failed.
 *
 * The actual result of the parallel task depends on its policy:
 * - Policy.Sequence: the parallel task fails as soon as one child fails; if all
 * children succeed, then the parallel task succeeds. This is the default policy.
 * - Policy.Selector: the parallel task succeeds as soon as one child succeeds;
 * if all children fail, then the parallel task fails.
 *
 * The typical use case: make the game entity react on event while sleeping or wandering.
 */
var Parallel = /** @class */ (function (_super) {
    __extends(Parallel, _super);
    function Parallel() {
        var _this = _super.call(this) || this;
        _this.orchestrator = new ParallelOrchestratorResume_1.ParallelOrchestratorResume();
        _this.policy = new ParallelPolicySequence_1.ParallelPolicySequence();
        _this.noRunningTasks = true;
        _this.currentChildIndex = 0;
        _this.lastResult = ParallelResult_1.ParallelResult.Running;
        return _this;
    }
    Parallel.prototype.run = function () {
        this.orchestrator.execute(this);
    };
    Parallel.prototype.childSuccess = function (runningTask) {
        this.lastResult = this.policy.onChildSuccess(this);
    };
    Parallel.prototype.childFail = function (runningTask) {
        this.lastResult = this.policy.onChildFail(this);
    };
    Parallel.prototype.childRunning = function (runningTask, reporter) {
        this.noRunningTasks = false;
    };
    Parallel.prototype.resetTask = function () {
        _super.prototype.resetTask.call(this);
        this.noRunningTasks = true;
    };
    Parallel.prototype.reset = function () {
        this.policy = new ParallelPolicySequence_1.ParallelPolicySequence();
        this.orchestrator = new ParallelOrchestratorResume_1.ParallelOrchestratorResume();
        this.noRunningTasks = true;
        this.lastResult = ParallelResult_1.ParallelResult.Running;
    };
    Parallel.prototype.resetAllChildren = function () {
        for (var i = 0, n = this.getChildCount(); i < n; ++i) {
            var child = this.getChild(i);
            child.reset();
        }
    };
    return Parallel;
}(BranchTask_1.BranchTask));
exports.Parallel = Parallel;
