"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var TaskStatus_1 = require("../TaskStatus");
var ParallelResult_1 = require("./ParallelResult");
/**
 * Children execute until they succeed or fail but will not re-run until
 * the parallel task has succeeded or failed.
 */
var ParallelOrchestratorJoin = /** @class */ (function () {
    function ParallelOrchestratorJoin() {
    }
    ParallelOrchestratorJoin.prototype.execute = function (parallel) {
        parallel.noRunningTasks = true;
        parallel.lastResult = ParallelResult_1.ParallelResult.Running;
        var n = parallel.getChildCount();
        for (parallel.currentChildIndex = 0; parallel.currentChildIndex < n; ++parallel.currentChildIndex) {
            var child = parallel.getChild(parallel.currentChildIndex);
            switch (child.getStatus()) {
                case TaskStatus_1.TaskStatus.Running:
                    child.run();
                    break;
                case TaskStatus_1.TaskStatus.Succeeded:
                case TaskStatus_1.TaskStatus.Failed:
                    break;
                default:
                    child.setControl(parallel);
                    child.start();
                    if (child.checkGuard(parallel)) {
                        child.run();
                    }
                    else {
                        child.fail();
                    }
                    break;
            }
            if (parallel.lastResult !== ParallelResult_1.ParallelResult.Running) {
                // Current child has finished either with success or fail.
                parallel.cancelRunningChildren(parallel.noRunningTasks ? parallel.currentChildIndex + 1 : 0);
                parallel.resetAllChildren();
                if (parallel.lastResult !== ParallelResult_1.ParallelResult.Running) {
                    parallel.success();
                }
                else {
                    parallel.fail();
                }
                return;
            }
        }
        parallel.running();
    };
    return ParallelOrchestratorJoin;
}());
exports.ParallelOrchestratorJoin = ParallelOrchestratorJoin;
